class APIEndpointConst:
    CONNECT = "connect"
    GET_TRIPLES = "get_triples"
    UPDATE_TRIPLES = "update_triples"
    CLASS_FRAME = "class_frame"
    WOQL_QUERY = "woql_query"
    DELETE_DATABASE = "delete_database"
    CREATE_DATABASE = "create_database"
    CREATE_GRAPH = ("create_graph",)
    DELETE_GRAPH = ("delete_graph",)
    CLONE = ("clone",)
    PUSH = ("push",)
    FETCH = ("fetch",)
    PULL = ("pull",)
    REBASE = ("rebase",)
    BRANCH = "branch"
