WOQL_TRIM_JSON = {
    "@type": "woql:Trim",
    "woql:untrimmed": {
        "@type": "woql:Datatype",
        "woql:datatype": {"@type": "xsd:string", "@value": "a"},
    },
    "woql:trimmed": {
        "@type": "woql:Datatype",
        "woql:datatype": {"@type": "xsd:string", "@value": "b"},
    },
}
