WOQL_CAST_JSON = {
    "@type": "woql:Typecast",
    "woql:typecast_value": {
        "@type": "woql:Variable",
        "woql:variable_name": {"@value": "Duration", "@type": "xsd:string"},
    },
    "woql:typecast_type": {"@type": "woql:Node", "woql:node": "xsd:integer"},
    "woql:typecast_result": {
        "@type": "woql:Variable",
        "woql:variable_name": {"@value": "Duration_Cast", "@type": "xsd:string"},
    },
}
