db_record_obj = {
    "@id": "doc:Database%5fadmin%7Caaaaaa",
    "@type": "terminus:Database",
    "rdfs:comment": {"@language": "en", "@value": "dasdasdds"},
    "rdfs:label": [
        {"@language": "en", "@value": "admin|aaaaaa"},
        {"@language": "en", "@value": "dadsds"},
    ],
    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
    "terminus:database_state": {
        "@id": "terminus:finalized",
        "@type": "terminus:DatabaseState",
    },
    "terminus:resource_name": {"@type": "xsd:string", "@value": "admin|aaaaaa"},
    "terminus:authority": [
        "terminus:branch",
        "terminus:class_frame",
        "terminus:clone",
        "terminus:commit_read_access",
        "terminus:commit_write_access",
        "terminus:create_database",
        "terminus:delete_database",
        "terminus:fetch",
        "terminus:inference_read_access",
        "terminus:inference_write_access",
        "terminus:instance_read_access",
        "terminus:instance_write_access",
        "terminus:manage",
        "terminus:meta_read_access",
        "terminus:meta_write_access",
        "terminus:push",
        "terminus:rebase",
        "terminus:schema_read_access",
        "terminus:schema_write_access",
    ],
}
