WoqlRe = {
    "@type": "woql:Regexp",
    "woql:pattern": {
        "@type": "woql:Datatype",
        "woql:datatype": {"@type": "xsd:string", "@value": ".*"},
    },
    "woql:regexp_string": {
        "@type": "woql:Variable",
        "woql:variable_name": {"@value": "string", "@type": "xsd:string"},
    },
    "woql:regexp_list": {
        "@type": "woql:Variable",
        "woql:variable_name": {"@value": "formated", "@type": "xsd:string"},
    },
}
